/* This model is the property of Cypress Semiconductor Corp.
-- and is protected by the US copyright laws, any unauthorized
-- copying and distribution is prohibited.
-- Cypress reserves the right to change any of the functional 
-- specifications without any prior notice.
-- Cypress is not liable for any damages which may result from
-- the use of this functional model
----------------------------------------------------------------------
-- File name : FM16W08.v
----------------------------------------------------------------------
-- Functionality : Verilog behavourial Model for FM16W08 
-- Source:  CYPRESS Data Sheet : "FM16W08 64Kbit F-RAM Memory" 
-----------------------------------------------------------------------
-- Developed by CYPRESS SEMICONDUCTOR
-- version |     author       | mod date | changes made
--    1.0         MEDU          03/27/14    New Model
--
-- Added a delayed ce_ signal for setup/hold time violation detection.
------------------------------------------------------------------------
-- PART DESCRIPTION :
-- Part:        FM16W08
--
-- Descripton:  Verilog behavourial Model  for  FM16W08
------------------------------------------------------------------------ */  

`timescale 1ns / 10ps       // simulator unit/precision

`include "config.v"

// Verilog model of FM16W08 - 8Kx8 F-RAM

module FM16W08(dq,addr,ce_,we_,oe_);
  inout [7:0] dq;
  input [12:0] addr;
  input ce_, we_, oe_;


//******************** Timing section ********************
reg notifier;
wire Enabled =  ~ce_;
reg   addr_change_reg = 1'b1;
wire  addr_change;
reg ce_stop_reg = 1'b1;
reg ce_start_reg = 1'b0;
wire ce_stop, ce_start;
reg [12:0] addr_prev;
reg [12:0] addr_prev1;
reg page_mode;
reg [7:0] data_reg = 'hz;
wire delayed_ce_; 


reg [31:0] i;

wire page_mode_ce = page_mode && Enabled;

assign #0.01 delayed_ce_ = ce_;


always @(addr)
begin
   addr_change_reg <= 1'b1;
    #1;
   addr_change_reg <= 1'b0; 
end

assign addr_change = (page_mode == 1'b1) ? 1'b0 : addr_change_reg;

specify
$period(negedge ce_,`Trc,notifier);               // CE cycle time check
$width(negedge ce_,`Tca,0,notifier);              // CE min low time check
$width(posedge ce_,`Tpc,0,notifier);              // CE min high time check
$period(negedge addr_change &&& Enabled,`Trc,notifier);       // Addr cycle time check
$setuphold(negedge delayed_ce_,addr,`Tas,`Tah,notifier);  // Addr setup/hold check
$width(negedge we_  &&& Enabled,`Twp,0,notifier);             // WE min low time check
$setuphold(posedge we_ &&& Enabled,dq,`Tds,`Tdh,notifier);    // Data setup/hold time check
$setuphold(posedge ce_ &&& ~we_,dq,`Tds,`Tdh,notifier);       // Data setup/hold time check
$recovery(negedge ce_,posedge we_,`Tcw,notifier);        // Write access time check

(addr[12:2] *> dq) =  (`Tce,`Tce,         // 0 -> 1 and 1 -> 0 transition
            `Thz,            // 0 -> z transition
            `Tce,            // z -> 1 transition
            `Thz,            // 1 -> z transition
            `Tce);           // z -> 0 transition
            
(ce_ *> dq) =  (`Tdummy,`Tdummy,  // 0 -> 1 and 1 -> 0 transition
            `Thz,            // 0 -> z transition
            `Tce,            // z -> 1 transition
            `Thz,            // 1 -> z transition
            `Tce);           // z -> 0 transition

(we_ *> dq) =  (`Tdummy,`Tdummy,  // 0 -> 1 and 1 -> 0 transition
            `Twz,            // 0 -> z transition
            `Twx,            // z -> 1 transition
            `Twz,            // 1 -> z transition
            `Twx);           // z -> 0 transition

(oe_ *> dq) =  (`Tdummy,`Tdummy,  // 0 -> 1 and 1 -> 0 transition
            `Tohz,           // 0 -> z transition
            `Toe,            // z -> 1 transition
            `Tohz,           // 1 -> z transition
            `Toe);           // z -> 0 transition

endspecify

//******************** End of Timing section ********************


//     Declare the RAM array and some signals

reg [7:0]  Mem [0:'h01fff];            // F-RAM array, 8Kx8

wire      READ  = (!ce_ && we_);
wire      WRITE = (!ce_ && !we_);
wire      OE = !oe_ ;

wire [7:0] data  = READ? data_reg : 8'bz;

initial
    begin
    `ifdef initMemFile
        //
        // memory initialization with data from file
        //
        $readmemh(`initMemFile,Mem);
        $display("Simulated memory array initialization with data from %s...",`initMemFile);
    `else
        //
        // memory initialization with ff
        //
        for(i=0; i<'h02000; i=i+1)
            begin
               Mem[i] = 8'hff;
            end
        $display("Simulated memory array initialization with 8'hff...");
    `endif
end    


always @(negedge WRITE)
begin
   if(ce_start == 1'b1)
   begin
      Mem[addr][7:0]  = dq[7:0];
      $display("write - %h  to Addr %h", dq[7:0], addr);
      ce_start_reg  <= 1'b0;                   
   end
   else
      $display("WARNING: /CE LOW violation during WRITE !!");
end

always @(ce_)
begin
   if(ce_)
   begin
     ce_start_reg <= 1'b0;
     ce_stop_reg  <= 1'b1;
   end
   else
   begin
     ce_start_reg <= 1'b1;
     ce_stop_reg  <= 1'b0;     
   end
end

assign ce_start = ce_start_reg;
assign ce_stop  = ce_stop_reg;


always @(negedge oe_)
begin
   if(READ)
   begin
      #`Toe data_reg = Mem[addr];
   end
end

always @(negedge ce_)
begin
   if(we_ && !oe_)
   begin
      #`Tce data_reg = Mem[addr];
      
   end
end

bufif1 b07 (dq[07],data[07],OE);
bufif1 b06 (dq[06],data[06],OE);
bufif1 b05 (dq[05],data[05],OE);
bufif1 b04 (dq[04],data[04],OE);
bufif1 b03 (dq[03],data[03],OE);
bufif1 b02 (dq[02],data[02],OE);
bufif1 b01 (dq[01],data[01],OE);
bufif1 b00 (dq[00],data[00],OE);

endmodule
