`timescale 1ns / 1ps

/* This model is the property of Cypress Semiconductor Corp.
-- and is protected by the US copyright laws, any unauthorized
-- copying and distribution is prohibited.
-- Cypress reserves the right to change any of the functional 
-- specifications without any prior notice.
-- Cypress is not liable for any damages which may result from
-- the use of this functional model
----------------------------------------------------------------------
-- File name : FM16W08_tb.v
----------------------------------------------------------------------
-- Functionality : Test Bench for Verilog behavourial Model of FM16W08 
-- Source:  CYPRESS Data Sheet : "FM16W08 64Kbit F-RAM Memory" 
-----------------------------------------------------------------------
-- Developed by CYPRESS SEMICONDUCTOR
--
-- version |     author       | mod date | changes made
--    1.0         MEDU          03/27/14    New Model
------------------------------------------------------------------------
-- PART DESCRIPTION :
-- Part:        FM16W08
--
-- Descripton:  Verilog behavourial Model Test Bench for  FM16W08
------------------------------------------------------------------------
-- NOTE:
-- The test bench checks design rules and throws error messages at certain instances. 
-- Ignore the first WARNING (WARNING: /CE LOW violation during WRITE!!). 
-- Test Bench starts after the message "Test Bench Simulation Starting...".
-- Test Bench should be run for 80us to cover all test cases.
------------------------------------------------------------------------*/  
`include "config.v"


module FM16W08_tb_v;

    // Inputs
    reg [12 :0] addr;
    reg ce_ = 1'b1;
    reg we_ = 1'b1;
    reg oe_ = 1'b1;
    
    reg [7:0] d_00;
    reg [7:0] d_01;
    reg [7:0] d_02;
    reg [7:0] d_03;
    reg [7:0] d_04;
    reg [7:0] d_05;
    reg [7:0] d_06;
    reg [7:0] d_07;
    reg [7:0] d_08;    

    // Bidirs
    wire [7:0] dq;
    reg  [7:0] dq_reg;

    // Instantiate the Unit Under Test (UUT)
    FM16W08 uut (
        .dq(dq), 
        .addr(addr), 
        .ce_(ce_), 
        .we_(we_), 
        .oe_(oe_)
        );

   assign dq = dq_reg;

    initial begin
        // Initialize Inputs
        addr = 0;
        ce_ = 1'b1;
        we_ = 1'b1;
        oe_ = 1'b1;
        dq_reg[7:0] <= 'hz;            

        // Wait 100 ns for global reset to finish
        #200;

        $display("\n\n*********************************");        
        $display("Test Bench Simulation Starting...");
        $display("*********************************\n");                

//**********************************
//      /WE Controlled Write
//**********************************        
        $display("\n\n*********************************");                        
        $display("Test case 1: /WE write Addr = 'h1F00  Data = 'h34");
        $display("*********************************\n");                        
        addr = 'h1F00;
        dq_reg[7:0] = 'h34;
        #`tAS;
        ce_ = 1'b0;
        #`Tce;
        we_ = 1'b0;
        #`tWZ;
        #`tWZ;
        #(`tDS+`tDH);
        we_ = 1'b1;
        #100;
        ce_ = 1'b1;
       
        #500;

//**********************************
//      /CE Controlled Write
//**********************************   
        $display("\n\n*********************************");                        
        $display("Test case 2: /CE write Addr = 'h0F00  Data = 'ha5");
        $display("*********************************\n");                                
        addr = 'h0F00;
        dq_reg[7:0] = 'ha5;      
        #`tAS;
        we_ = 1'b0;
        ce_ = 1'b0;
        #100;
        #(`tDS+`tDH);
        ce_ = 1'b1;
        #100;
        we_ = 1'b1;
        
        #500;

//**********************************
//      /OE READ
//**********************************
        $display("\n\n*********************************");                        
        $display("Test case 3: /OE Read Addr = 'h1F00  Expected Data = 'h34");
        $display("*********************************\n");                                        
        
        dq_reg[7:0] <= 'hz;        
        addr = 'h1F00;
        #`tAS;
        ce_ = 1'b0;
        #100;
        #10;
        oe_ = 1'b0;
        #`Toe;
        #2;
        $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);
        #200;
        oe_ = 1'b1;
        ce_ = 1'b1;
        
        #500;
//**********************************
//      /CE READ
//**********************************
        $display("\n\n*********************************");                        
        $display("Test case 4: /CE Read Addr = 'h0F00  Expected Data = 'ha5");
        $display("*********************************\n");                                        
        dq_reg[7:0] <= 'hz;        
        addr = 'h0F00;
        #`tAS;
        oe_ = 1'b0;
        #100;
        #10;
        ce_ = 1'b0;
        #`Tce;
        #2;        
        $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);        
        addr = 'h7F00;
        #`Tce;
        #2;        
        $display("Addr enabled read not supported : F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);        
        #200;      
        oe_ = 1'b1;
        ce_ = 1'b1;
        
        #500

//*******************************************
//   Write data to 9 continuous locations
//*******************************************
    $display("\n\n*********************************");                        
    $display("Test case 5: Write 01 to 09 from Addr = 'h00000");
    $display("*********************************\n");                                

    FRAM_write_we('h00000, 'h01);
    FRAM_write_we('h00001, 'h02);
    FRAM_write_we('h00002, 'h03);
    FRAM_write_we('h00003, 'h04);
    FRAM_write_we('h00004, 'h05);
    FRAM_write_we('h00005, 'h06);
    FRAM_write_we('h00006, 'h07);
    FRAM_write_we('h00007, 'h08);
    FRAM_write_we('h00008, 'h09);

//*******************************************
//   /OE Read from 9 continuous locations
//*******************************************
    $display("\n\n*********************************");                        
    $display("Test case 6: Read 9 bytes from Addr = 'h00000");
    $display("*********************************\n");  
    
    FRAM_read_oe('h00000, d_00);
    $display("F-RAM /OE Read: 'h00000, Data - %h", d_00);
    
    FRAM_read_oe('h00001, d_01);
    $display("F-RAM /OE Read: 'h00001, Data - %h", d_01);
    
    FRAM_read_oe('h00002, d_02);
    $display("F-RAM /OE Read: 'h00002, Data - %h", d_02);
    
    FRAM_read_oe('h00003, d_03);
    $display("F-RAM /OE Read: 'h00003, Data - %h", d_03);
    
    FRAM_read_oe('h00004, d_04);
    $display("F-RAM /OE Read: 'h00004, Data - %h", d_04);
    
    FRAM_read_oe('h00005, d_05);
    $display("F-RAM /OE Read: 'h00005, Data - %h", d_05);
    
    FRAM_read_oe('h00006, d_06);
    $display("F-RAM /OE Read: 'h00006, Data - %h", d_06);
    
    FRAM_read_oe('h00007, d_07);
    $display("F-RAM /OE Read: 'h00007, Data - %h", d_07);
    
    FRAM_read_oe('h00008, d_08);
    $display("F-RAM /OE Read: 'h00008, Data - %h", d_08);

    $display("*********************************\n");  
    
    end

task FRAM_read_oe;
input [12 :0] t_addr;
output [7:0] t_dq;
begin
   dq_reg[7:0] <= 'hz;        
   addr = t_addr;
   #`tAS;
   ce_ = 1'b0;
   #100;
   #10;
   oe_ = 1'b0;
   #`Toe;
   #2;
   t_dq = dq;
   #200;
   oe_ = 1'b1;
   ce_ = 1'b1;
       
   #500;
end	
endtask

task FRAM_soft_read;
input [12 :0] t_addr;
begin
   $display("Soft Read");
   oe_ = 1'b1;        
   dq_reg[7:0] <= 'hz;        
   addr = t_addr;
   #`tAS;
   ce_ = 1'b0;
   #100;
   #10;
   #`Toe;
   #2;
   #200;
   oe_ = 1'b1;
   ce_ = 1'b1;
       
   #500;
end	
endtask

task FRAM_write_we;
input [12 :0] t_addr;
input [7:0] t_dq;
begin
   addr = t_addr;
   dq_reg[7:0] = t_dq;
   #`tAS;
   ce_ = 1'b0;
   #`Tce;
   we_ = 1'b0;
   #`tWZ;
   #`tWZ;
   #(`tDS+`tDH);
   we_ = 1'b1;
   #100;
   ce_ = 1'b1;
       
   #500;
end
endtask


task FRAM_read_oe_addr;
input [12 :0] t_addr;
input [7:0] t_len;
reg   [7:0] i;
begin
   dq_reg[7:0] <= 'hz;        
   addr = t_addr;
   #`tAS;
   ce_ = 1'b0;
   #100;
   #10;
   oe_ = 1'b0;
   #`Toe;
   #2;
   $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);
   #200;
   for(i=1; i<t_len; i= i+1)
   begin
      addr = t_addr + i;
      #`Trc;
      #2;
      $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);
      #200;        
   end

   oe_ = 1'b1;
   ce_ = 1'b1;
end
endtask

task FRAM_read_ce_addr;
input [12 :0] t_addr;
input [7:0] t_len;
reg   [7:0] i;
begin
   dq_reg[7:0] <= 'hz;        
   addr = t_addr;
   #`tAS;
   oe_ = 1'b0;
   #100;
   #10;
   ce_ = 1'b0;
   #`Tce;
   #2;        
   $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);  
   #200;
   for(i=1; i<t_len; i= i+1)
   begin
      addr = t_addr + i;
      #`Trc;
      #2;
      $display("F-RAM Read: Addr - %h, Data - %h", addr, dq[7:0]);
      #200;        
   end

   ce_ = 1'b1;
   oe_ = 1'b1;
end
endtask

        
endmodule

