
//clk:50MHz
module fram_rw(FRAM_rd_req,FRAM_wr_req,//request signal:input
			   addr_in,data_in,data_out,//address and data input from top level
			   FRAM_data,FRAM_addr,//Data and Address bus connect to FRAM
			   FRAM_ce,FRAM_we,FRAM_oe,//FRAM Control signal.Chip Enable,Write Enable and Data Output Enable.
			   rddata_rdy,wr_completed,//Ready and Completed signal.Feedback to the top level
			   clk,rst_n//clock and reset for this module
			   );
input FRAM_rd_req;//read request
input FRAM_wr_req;//write request
input[7:0] data_in;
input[12:0] addr_in;
input clk;
input rst_n;
inout[7:0] FRAM_data;//data bus,connect to FRAM data pin
output [12:0] FRAM_addr;//address bus,connect to FRAM address pin.
output FRAM_ce;//Chip Enable
output FRAM_we;//Write enable
output FRAM_oe;//Data output enable
output rddata_rdy;//data read from Fram is ready
output wr_completed;//data write cycle is completed
output[7:0] data_out;

reg[12:0] FRAM_addr_r;//Address output register
reg FRAM_ce_r;
reg FRAM_we_r;
reg FRAM_oe_r;
reg rddata_rdy_r;
reg wr_completed_r;
//																									__________________
//												  											 |				  				|				
reg FRAM_data_dir;//FRAM databus direction control.| 1:output 0:input |
//												   											 |__________________|
//
//IO signal assignment
assign data_out=FRAM_data_dir ? 8'hzz:FRAM_data;
assign FRAM_data=FRAM_data_dir ? data_in:8'hzz;
assign FRAM_ce=FRAM_ce_r;
assign FRAM_we=FRAM_we_r;
assign FRAM_oe=FRAM_oe_r;
assign FRAM_addr=FRAM_addr_r;

`define tCE_DLY (cnt_dly==4'h3)
`define tPC_DLY (cnt_dly==4'h2)
reg[2:0] cnt_dly;
reg cnt_en;

reg[3:0] cstate;
reg[3:0] nstate;

//Gray Code(Typical)
parameter IDLE=4'b0000;
parameter READ0=4'b0001;
parameter READ1=4'b0011;
parameter READ2=4'b0010;
parameter READ3=4'b0110;

parameter WRITE0=4'b0111;
parameter WRITE1=4'b0101;
parameter WRITE2=4'b0100;
parameter WRITE3=4'b1100;

always @(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		cstate<=IDLE;
	else
		cstate<=nstate;
end

always @(cstate or FRAM_rd_req or FRAM_wr_req or cnt_dly)
begin
	case(cstate)
		IDLE:
			begin
				if(FRAM_rd_req)
					nstate=READ0;
				else if(FRAM_wr_req)
					nstate=WRITE0;
				else
					nstate=IDLE;
			cnt_en=0;//disble the delay count(default make it to zero)
			end
		READ0:
				nstate=READ1;
		READ1:
			begin
				if(`tCE_DLY)
				begin
					nstate=READ2;//jump to READ2
					cnt_en=0;
				end
				else
				begin
					cnt_en=1;
					nstate=READ1;//still run in the current state
				end
			end
		READ2://read data
			nstate=READ3;
		READ3:
		begin
			if(`tPC_DLY)
			begin
				nstate=IDLE;
				cnt_en=0;
			end
			else
			begin
				nstate=READ3;
				cnt_en=1;
			end
		end
		WRITE0:
			nstate=WRITE1;
		WRITE1:
		begin
		if(`tCE_DLY)
		begin
			nstate=WRITE2;
			cnt_en=0;
		end
		else
		begin
			nstate=WRITE1;
			cnt_en=1;
		end
		end
		WRITE2:
			nstate=WRITE3;
		WRITE3:
		begin
			if(`tPC_DLY)
			begin
				nstate=IDLE;
				cnt_en=0;
			end
			else
			begin
				nstate=WRITE3;
				cnt_en=1;
			end
		end
	default:nstate=IDLE;
	endcase
end

// Address/control
always @(posedge clk or negedge rst_n)
begin
if(!rst_n)
begin
	FRAM_ce_r<=1'b1;
	FRAM_oe_r<=1'b1;//diable the output data of FRAM
	FRAM_we_r<=1'b1;//write disable
	FRAM_addr_r<=13'h1fff;
end
else
begin
	case(nstate)
		READ0:
		begin
			FRAM_ce_r<=1'b1;
			FRAM_oe_r<=1'b1;//diable the output data of FRAM
			FRAM_we_r<=1'b1;//write disable
			FRAM_addr_r<=addr_in;//prepare the address
		end
		READ1,READ2://pull down CEto latch the Address
		begin
			FRAM_ce_r<=1'b0;//
			FRAM_oe_r<=1'b0;//wait the valid data on the bus
			FRAM_we_r<=1'b1;//write disable
			FRAM_addr_r<=addr_in;//address is innored in this step,will it synchronized to latch?
		end
		READ3://pull up the CE and OE
		begin
			FRAM_ce_r<=1'b1;
			FRAM_oe_r<=1'b1;
			FRAM_we_r<=1'b1;
			FRAM_addr_r<=addr_in;
		end
		WRITE0:
		begin
			FRAM_ce_r<=1'b1;
			FRAM_oe_r<=1'b1;
			FRAM_we_r<=1'b0;
			FRAM_addr_r<=addr_in;
		end
		WRITE1,WRITE2:
		begin
			FRAM_ce_r<=1'b0;
			FRAM_oe_r<=1'b1;
			FRAM_we_r<=1'b0;
			FRAM_addr_r<=addr_in;			
		end
		WRITE3:
		begin
			FRAM_ce_r<=1'b1;
			FRAM_oe_r<=1'b1;
			FRAM_we_r<=1'b1;
//			FRAM_addr_r<=addr_in;		
		end
	default:
	begin
		FRAM_ce_r<=1'b1;
		FRAM_oe_r<=1'b1;//diable the output data of FRAM
		FRAM_we_r<=1'b1;//write disable
		FRAM_addr_r<=13'hzzz;
	end
	endcase
end
end

assign wr_completed=wr_completed_r;
always @(cstate)
begin
	case(cstate)
		WRITE3:
			wr_completed_r<=1'b1;
		default:
			wr_completed_r<=1'b0;
	endcase
end
assign rddata_rdy=rddata_rdy_r;
always @(cstate)
begin
	case(cstate)
	READ3:
		rddata_rdy_r<=1'b1;
	default:
		rddata_rdy_r<=1'b0;
	endcase
end

always @(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		cnt_dly<=3'h0;
	else if(cnt_en)
		cnt_dly<=cnt_dly+3'h1;
	else
		cnt_dly<=3'h0;
end

always @(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		FRAM_data_dir<=1'b0;
	else
	begin
		case(cstate)
			IDLE:if(FRAM_wr_req) FRAM_data_dir<=1'b1;
					 else if(FRAM_rd_req) FRAM_data_dir<=1'b0;
					 else FRAM_data_dir<=1'b0;
			WRITE0: FRAM_data_dir<=1'b1;
			WRITE1: FRAM_data_dir<=1'b1;
			WRITE2: FRAM_data_dir<=1'b1;
			WRITE3: FRAM_data_dir<=1'b1;
			default:FRAM_data_dir<=1'b0;
		endcase
	end
end





endmodule

